<?php

affiliate_pages_pro_use([
  'Controllers/Api/ApiController',
  'Models/Page',
  'Models/Block',
  'Models/Visit',
  'Models/Split',
  'Models/Stat',
  'Helpers/Parses/ParseItem'
]);

class AffiliatePagesPro_ApiPageController extends AffiliatePagesPro_ApiController
{
  use AffiliatePagesPro_ParseItem;

  protected $expire_access_seconds = 0;

  protected $time_u;

  protected $is_preview = false;

  protected $default_ogs = [
    'fb_app_id' => '2248016688855357'
  ];

  protected $page_fields = [
    'identifier',
    'id',
    // 'user_id',
    // 'page_type',
    'title',
    'des',
    'image',
    'theme_base',
    'affiliate_link',
    'affiliate_dash',
    'theme_color',
    'text_color',
    'is_dark',
    'bg_color',
    'bg_image',
    'bg_imagefx',
    'bg_imageblur',

    'spacing_x',
		'spacing_y',

    'og_title',
    'og_des',
    'og_keywords',
    'og_img',
    'og_imgw',
    'og_imgh',
    'slug',

    'clicks',

    'tracking_active',
    'tracking_header',
    'tracking_footer',
    'tracking_analytics',
    'tracking_fbpixel',

    // video fields
    'is_active',

    'padding_x',
    'padding_y',
    'margin',
    'margin_x',
    'margin_y',

    'padding',
    'shadow',


    'content_bg',

    'footer_active',
    'footer_color',
    'footer_menu',
    'footer_facebook',
    'footer_twitter',
    'footer_linkedin',
    'footer_instagram',
    'footer_youtube',
    'footer_content',
    'cookies_notice',

    'topbar_active',
    'topbar_color',
    'topbar_textcolor',
    'topbar_title',
    'topbar_logo',
    'topbar_name',
    'topbar_avatar',
    'topbar_shadow',
    'topbar_namelink',
    'topbar_avatarlink',
    'topbar_social',

    'topbar_linktitle',
    'topbar_logolink',
    'topbar_flat',
    'max_width',
    'use_width',

    // 1.0.4
    // 'schedule_timezone',
		// 'schedule_start',
		// 'schedule_startdate',
		// 'schedule_starttime',
		// 'schedule_startu',
		// 'schedule_end',
		// 'schedule_enddate',
		// 'schedule_endtime',
		// 'schedule_endu',
		// 'redirect_start',
		// 'redirect_end',
		'border_radius',

		// v.1.0.11
		'bg_video',
    'bg_mask'

  ];

  protected $block_fields = [
    'identifier',
    // 'user_id',
    // 'page_id',
    'id',
    'block_type',
    // 'type_id',
    'video_id',
    'item_link',

    'title',
    'des',
    'content',

    'clicks',

    'sort',

    'video',
    'is_video',
    'start',
    'end',
    'autoplay',
    'hide_controls', // 0: no, 1: yes, 2: muted
    'poster',// (0: autohide, 1: always, 2: never )
    'poster_pause',
    'poster_end',
    'duration',
    'trim_active',
    'product_placement',
    'muted',
    'product_img',
    'auto_redirect',
    'disable_fullscreen',
    'disable_mask',

    'url',
    'target',
    'is_dark',
    'color',
    'style',
    'item_style',
    'size',

    'margin',
    'margin_x',
    'margin_y',
    'padding',
    'use_margin',
    'use_padding',
    'text_color',
    'shadow',
    'item_shadow',
    'bg_color',
    'bg_image',
    'source',

    'flex_xs',
    'flex_sm',
    'flex_md',
    'flex_lg',
    'align',

    'item_height',
    'item_width',
    'item_contain',
    'aspect_ratio',
    'custom_size',

    'is_split',
    'split_index',

    // v1.0.4
		'schedule_timezone',
		'schedule_start',
    'schedule_startdate',
    'schedule_starttime',
    'schedule_startu',
    'schedule_end',
    'schedule_enddate',
    'schedule_endtime',
    'schedule_endu',
    'hide_ended',
		'item_borderradius',
    'border_radius',
    'title_color',
    'color2',
		'spacing_x',
		'spacing_y',
    'item_startu',
		'expanded',
		'show_more',
		'left_icon',
		'left_iconcolor',
		'right_icon',
		'right_iconcolor',

    // v1.0.10
    'item_block'

  ]; // / block fields

  protected $split_fields = [
    'identifier',
    'id',
		'user_id',
		'page_id',
		'block_id',

		'block_type',

		'content',

		'video',
		'is_video',
		'start',
		'end',
		'autoplay',
		'hide_controls', // 0: no, 1: yes, 2: muted
		'poster',// (0: autohide, 1: always, 2: never )
		'poster_pause',
		'poster_end',
		'duration',
		'trim_active',
		'product_placement',
		'muted',
		'product_img',
		'auto_redirect',
		'disable_fullscreen',
		'disable_mask',

		'url',
		'target',
		'is_dark',
		'color',
		'style',
		'item_style',
		'size',
		'item_link',

		'margin',
		'margin_x',
		'margin_y',
		'padding',
		'use_margin',
		'use_padding',
		'text_color',
		'shadow',
		'item_shadow',
		'bg_color',
		'bg_image',
		'source',

		'flex_xs',
		'flex_sm',
		'flex_md',
		'flex_lg',
		'align',

		'item_height',
		'item_width',
		'item_contain',
		'aspect_ratio',
		'custom_size',
    'hide_ended',
		'item_borderradius',
    'border_radius',
    'title_color',
    'color2',
		'spacing_x',
		'spacing_y',
    'item_startu',
		'expanded',
		'show_more',
		'left_icon',
		'left_iconcolor',
		'right_icon',
		'right_iconcolor',

    // v1.0.10
    'item_block'
  ];

  protected $page_defaults = [
      'topbar_title' => 'title'
  ];

  public function __construct()
  {
    $this->time_u = current_time('U');
  }

  public function preview($request) {

    if(empty(self::$debug)) {

      if(!is_user_logged_in()) {
        return;
      }

      if(!current_user_can('administrator')) {
        return;
      }

    }

    $this->is_preview = true;

    // set scheduled time
    $count = 0;
    foreach(['date', 'time', 'timezone'] as $i) {
      if(!empty($request->{$i})) {
        $count++;
      }
    }

    if($count > 2) {
      date_default_timezone_set($request->timezone);
      $this->time_u = strtotime($request->date.' '.$request->time.':00');
    }

    $this->index($request);
  }

  public function index($request)
  {
    if(empty($this->is_preview)) {

      $visit = AffiliatePagesPro_Visit::where('identifier', $request->visit)->first(['id', 'page_id', 'created_ts']);

      // if(empty($visit)) { // or expired
      //   return;
      // }
      // elseif(!empty($this->expire_access_seconds) && ((int)$visit->created_ts < (current_time('U') - $this->expire_access_seconds))) {
      //   return;
      // }

      if(!empty($visit)) {
        $page = AffiliatePagesPro_Page::find($visit->page_id, $this->page_fields);
      }


    } // / isPreview
          // echo 'preview'.$request->page; exit;

    // next attempt slug
    if(empty($page)) {
      $page = AffiliatePagesPro_Page::where('slug', $request->page)->first($this->page_fields);
    }
    // wp_send_json([$request, $page]); exit;

    if(empty($page)) {
      return;
    }

    $page = $this->parseItem($page);

    $options = self::$options;

    if(empty($options)) {
      $options = [];
    }


    // wp_send_json($options); exit;
    // topbar_avatarlink

    $data = affiliate_pages_pro_obj_atts($page, $options);

    // wp_send_json($data); exit;

    foreach($this->page_defaults as $key => $value) {
      if(empty($data->{$key}) && !empty($data->{$value})) {
        $data->{$key} = $data->{$value};
      }
    }

    $blocks = AffiliatePagesPro_Block::where('page_id', $page->id)->orderBy('sort')->get($this->block_fields);

    // wp_send_json($blocks[0]); exit;

    $removeBlocks = [];

    if(!empty($blocks)) {

      //
      $count = 0;
      foreach($blocks as $block) {

        // scheduled blocks
        if($this->isScheduledHide($block)) {
          array_push($removeBlocks, $count);
        }

        // active split test
        else {

          if(!empty($block->is_split)) {

            if(empty($this->is_preview)) {
              $stat = new AffiliatePagesPro_Stat;
              $stat->visit_id = $visit->id;
              $stat->page_id = $page->id;
              $stat->block_id = $block->id;
              $stat->is_visit = 1;
            }

            //  || (int) $block->split_index == 0
            if(!isset($block->split_index)) {

              $split_index = 0; // start with split next
            }
            else {
              $split_index = (int) $block->split_index;

              $split = $this->getSplit($block->id, $split_index);

              // back to zero
              if(empty($split)) {
                $split_index = 0;
              }
              else {
                $blocks[$count] = $split;

                if(empty($this->is_preview)) {
                  $stat->split_id = $split->id;
                }
                $split_index++;
              }

            }

            // update block split index
            if(empty($this->is_preview)) {

              $b = AffiliatePagesPro_Block::find($block->id, ['id']);
              if(!empty($b)) {
                $b->split_index = $split_index;
                $b->save();
              }
              // save stats
              $stat->save();
            }


          } // is split ended

          $block = $blocks[$count];

          // set item_starts_in in
          if(!empty($blocks[$count]->item_startu)) {
            $blocks[$count]->item_starts_in = (int) $blocks[$count]->item_startu - $this->time_u;
            if($blocks[$count]->item_starts_in < 0 && !empty($blocks[$count]->hide_ended)) {
              array_push($removeBlocks, $count);
            }
          }

        }
        $count++;
      }
    }

    if(!empty($removeBlocks)) {
      $updated = []; $count = 0;
      foreach($blocks as $block) {
        if(!in_array($count, $removeBlocks)) {
          array_push($updated, $block);
        }
        $count++;
      }
      $blocks = $updated;
    }

    wp_send_json([
      'blocks' => $blocks,
      'data' => $data,
      'footer_menu' => $this->getPageFooterMenu($page),
      'footer_shares' => $this->getFooterShares($page),
      'cookies_notice' => $this->getCookiesNotice($page)
    ]);
    exit;

  }


  // see if scheduled and not showing now
  private function isScheduledHide($block) {

    // not active
    if(empty($block->schedule_timezone)) {
      return false;
    }
    if(empty($block->schedule_start) && empty($block->schedule_end)) {
      return false;
    }

    // is unset scheduled preview


    // not started
    if(!empty($block->schedule_start) && !empty($block->schedule_startu)) {
      $diff = $block->schedule_startu - $this->time_u;
      if($diff > 0) {
        return true;
      }
    }

    // ended
    if(!empty($block->schedule_end) && !empty($block->schedule_endu)) {
      $diff = $block->schedule_endu - $this->time_u;
      if($diff < 0) {
        return true;
      }
    }
    return false;
  }

  // split index starts at 1
  private function getSplit($block_id, $split_index = 0)
  {
    $split = AffiliatePagesPro_Split::where('block_id', $block_id)
    ->page($split_index+1)
    ->take(1)
    ->get($this->split_fields);
    if(!empty($split)) {
      $split = $this->parseItem($split);
      return $split{0};
    }
    return null;
  }

  private function getPage($slug) {

    $page = AffiliatePagesPro_Page::where('slug', $slug)->first($this->page_fields);

    if(empty($page)) {
      return;
    }

    $page = $this->parseItem($page);

    $options = self::$options;

    if(empty($options)) {
      $options = [];
    }

    $data = affiliate_pages_pro_obj_atts($page, $options);


    foreach($this->page_defaults as $key => $value) {
      if(empty($data->{$key}) && !empty($data->{$value})) {
        $data->{$key} = $data->{$value};
      }
    }

    $blocks = AffiliatePagesPro_Block::where('page_id', $page->id)->orderBy('sort')->get($this->block_fields);


    return [
      'blocks' => $blocks,
      'data' => $data,
      'footer_menu' => $this->getPageFooterMenu($page),
      'footer_shares' => $this->getFooterShares($page),
      'cookies_notice' => $this->getCookiesNotice($page)
      ];

  }

  private function getCookiesNotice($page) {
    if(empty($page->cookies_notice)) {
      return [];
    }

    $args = [
      'notice' => 'We use cookies',
      'thanks' => 'Thank you!',
      'button' => 'OK',
      'placement' => 'bottom'
    ];

    foreach($args as $key => $value) {
      if(!empty(self::$options['cookies_'.$key])) {
        $args[$key] = self::$options['cookies_'.$key];
      }
    }

    return $args;
  }

  private function getFooterShares($page) {

    $shares = [];

    $socials = [
      'facebook' => 'fb',
      'twitter' => 'tw',
      'linkedin' => 'li',
      'instagram' => 'in',
      'youtube' => 'yt'
    ];


    foreach($socials as $key => $value) {

      // is active
      if(!empty($page->{'footer_'.$key})) {

        $entry = [
          'value' => $key
        ];

        if(!empty(self::$options) && !empty(self::$options['social_'.$value])) {
          $entry['url'] = self::$options['social_'.$value];
        }

        array_push($shares, $entry);
      }
    }

    return $shares;

  }

  private function getPageFooterMenu($page) {

    if(!empty($page->footer_active) && !empty($page->footer_menu)) {
      return $this->getMenu($page->footer_menu);
    }
    return [];
  }

  private function getMenu($slug)
  {
    $links = wp_get_nav_menu_items($slug);

    $menu = [];

    if(!empty($links)) {
      foreach($links as $link) {

        array_push($menu, [
          'url' => $link->url,
          'title' => $link->title,
          'target' => $link->target
        ]);
      }
    }

    return $menu;
  }



}
