<?php

affiliate_pages_pro_use([
  'Controllers/Api/ApiController',
  'Core/Time',
  'Models/Page',
  'Models/Visit',
  'Models/Click'
]);


class AffiliatePagesPro_ApiPageVisitsController extends AffiliatePagesPro_ApiController
{

  public function onReset($request)
  {
    $visits = new AffiliatePagesPro_Visit;
    $visits->wpDelete(['page_id' => $request->id]);

    $clicks = new AffiliatePagesPro_Click;
    $clicks->wpDelete(['page_id' => $request->id]);

    /// remove from page
    $page = AffiliatePagesPro_Page::where('id', $request->id)->first(['id']);
    if(!empty($page)) {
      $page->visits = 0;
      $page->visits_unique = 0;
      $page->save();
    }

      wp_send_json_success();
  }

  protected function getFirst($request)
  {
    return AffiliatePagesPro_Visit::where('page_id', $request->id)
                                    ->orderBy('created_ts', 'asc')
                                    ->first(['id','created_ts']);
  }

  protected function getVisits($page_id, $start_ts, $end_ts, $is_unique=null)
  {
    $q = AffiliatePagesPro_Visit::where('page_id', $page_id)
                                    ->andWhere('created_ts', '>', $start_ts)
                                    ->andWhere('created_ts', '<', $end_ts);
    if(!empty($is_unique)) {
      $q->andWhere('is_unique', 1);
    }

    return (int) $q->count();
  }

  protected function getClicks($page_id, $start_ts, $end_ts, $is_unique=null)
  {
    $q = AffiliatePagesPro_Click::where('page_id', $page_id)
                                    ->andWhere('created_ts', '>', $start_ts)
                                    ->andWhere('created_ts', '<', $end_ts);
    if(!empty($is_unique)) {
      $q->andWhere('is_unique', 1);
    }

    return (int) $q->count();
  }



  public function index($request)
  {

    $t = new AffiliatePagesPro_Time();

    if(empty($request->range)) {
      // 7 days
      $days = $t->getPastDays(7);
    }
    elseif(is_numeric($request->range)) {
      // x days
      $days = $t->getPastDays($request->range);
    }
    elseif($request->range == 'current_week') {
      $days = $t->getCurrentWeek();
    }
    elseif($request->range == 'current_month') {
      $days = $t->getCurrentMonth();
    }
    else { // == 'all'

      $first = $this->getFirst($request);

      $num = ceil(($t->ts - (int) $first->created_ts) / 86400);


      $days = $t->getPastDays($num);

    }



    $count = 0;
    $total = 0;
    $total_unique = 0;

    $clicks = 0;
    $clicks_unique = 0;

    foreach($days as $i) {

      $days[$count]['visits'] = $this->getVisits($request->id, $i['start_ts'], $i['end_ts']);

      $total += $days[$count]['visits'];

      $days[$count]['visits_unique'] = $this->getVisits($request->id, $i['start_ts'], $i['end_ts'], 1);

      $total_unique += $days[$count]['visits_unique'];


      // clicks
      $days[$count]['clicks'] = $this->getClicks($request->id, $i['start_ts'], $i['end_ts']);
      $clicks += $days[$count]['clicks'];

      $days[$count]['clicks_unique'] = $this->getClicks($request->id, $i['start_ts'], $i['end_ts'], 1);
      $clicks_unique += $days[$count]['clicks_unique'];

      $count++;

    }

    wp_send_json(['data' => $days, 'days' => count($days), 'visits' => $total, 'visits_unique' => $total_unique, 'clicks' => $clicks, 'clicks_unique' => $clicks_unique]);
  }
}
