<?php

affiliate_pages_pro_use([
  'Controllers/Controller',
  'Controllers/Database/DatabaseController',
  'Models/Page',
  'Models/Stat',
  'Helpers/Routes',
  'Helpers/Parses/ParseItem',
  'Packages/Google/Analytics/GoogleAnalytics',
  'Packages/Tracking/Tracking'
]);



class AffiliatePagesPro_PageController extends AffiliatePagesPro_Controller
{

  use AffiliatePagesPro_Routes, AffiliatePagesPro_ParseItem;

  // testing
  protected $dev_path = 'http://localhost:8082';

  protected $page_fields = [
    'id',
    'identifier',
    'title',
    'des',
    'favicon',
    'image',
    'theme_base',
    'affiliate_link',
    'affiliate_dash',
    'theme_color',
    'text_color',
    'is_dark',
    'bg_color',
    'bg_image',
    'bg_imagefx',
    'bg_imageblur',
    'tracking_analytics',
    'tracking_fbpixel',
    'tracking_header',
    'tracking_footer',
    'og_title',
    'og_des',
    'og_img',
    'og_imgw',
    'og_imgh',
    'visits',
    'visits_unique',
    'is_active',
    'no_index'
  ];

  protected $data_atts = [
    'theme_base',
    'theme_color',
    'text_color',
    'is_dark',
    'bg_color',
    'bg_image',
    'bg_imagefx',
    'bg_imageblur',
  ];

  protected $cdn = [
    'css' => [
      'https://fonts.googleapis.com/css?family=Roboto:100,300,400,500,700,900',
      'https://fonts.googleapis.com/css?family=Material+Icons',
      'https://use.fontawesome.com/releases/v5.8.2/css/all.css',
      'https://cdn.plyr.io/3.5.2/plyr.css'
    ],
    'js' => [
      'https://cdn.plyr.io/3.5.2/plyr.polyfilled.js'
    ]
  ];

  protected $default_options = [
    'fb_app_id' => 190569694855224
  ];

  protected $option_fallback = [
    'tracking_analytics',
    'tracking_fbpixel'
  ];

  protected $page_defaults = [
      // 'favicon' => 'image',
      'og_title' => 'title',
      'og_des' => 'des',
      'og_image' => 'image'
  ];

  public function preview($request) {

    if(!is_user_logged_in()) {
      return;
    }

    if(!current_user_can('administrator')) {
      return;
    }

    $this->index($request, true);
  }

  public function index($request, $isPreview=false)
  {
    $dbupdate = new AffiliatePagesPro_DatabaseController;
    $dbupdate->checkUpdate();

    $page = AffiliatePagesPro_Page::where('slug', $request->slug)->first($this->page_fields);

    if(empty($page)) {
      return;
    }

    $url = $this->prefixRoute($request->slug);

    if(empty($isPreview) && empty($page->is_active)) {
      $page->message = 'This page is not active.';
      $this->view('page.message', compact('page', 'url')); exit;
    }

    // add visit
    if(empty($isPreview)) {
      $visit = $page->addVisit();
    }

    // page defaults
    foreach($this->page_defaults as $key => $value) {
      if(empty($data->{$key}) && !empty($data->{$value})) {
        $data->{$key} = $data->{$value};
      }
    }



    $options = AffiliatePagesPro::$options;

    if(empty($options)) {
      $options = [];
    }

    foreach($this->default_options as $key => $value) {
      if(empty($options[$key])) {
        $options[$key] = $value;
      }
    }

    if(empty($page->favicon) && !empty($options['favicon'])) {
      $page->favicon = $options['favicon'];
    }

    foreach($this->option_fallback as $i) {
      if(empty($page->{$i}) && isset($options[$i]) == true) {
        $page->{$i} = $options[$i];
      }
    }



    $cdn = $this->cdn;

    // Data Atts

    $data = [
      'url' => $this->prefixRoute($request->slug)
    ];

    if(empty($isPreview)) {
      $data['token'] = $visit->identifier;
    } else {
      // can add preview user
      $data['token'] = 'preview';
      $data['preview'] = '1';
    }

    foreach($this->data_atts as $i) {
      if(!empty($page->{$i})) {
        $data[$i] = $page->{$i};
      }
    }

    if(!empty($isPreview)) {
      foreach(['date', 'time', 'timezone'] as $i) {
        if(!empty($request->{$i})) {
          $data[$i] = $request->{$i};
        }
      }
    }

    $page = $this->parseItem($page);

    $this->view('page.index', compact('page', 'url', 'options', 'cdn', 'data'));
    exit;
  }

}
