<?php

/*

August 25, 2018
adding update support
 *
 * May 1, 2018
 * adding new static function to create tables
 * addTable()
 *
 */

if(class_exists('AffiliatePagesPro_Migration')) {
    return;
}

abstract class AffiliatePagesPro_Migration extends AffiliatePagesPro
{
    private $sql = '';
    private $primary_key = 'id';

    // options
    protected $use_app_group_id = false;
    protected $use_app_id = true;
    protected $use_custom_id = null;

    protected $table = '';


    // install rows on new setup

    protected $rows;

    protected function getModel() {}

    public static function addTable($table, $name=null, $mod=null)
    {
        if(empty($name)) {
            $name = ucfirst($table);
        }

         affiliate_pages_pro_use('../database/tables/'.$table, $mod);

        $name = self::getPrefix().'DB_'.$name;
        (new $name())->up();


    }

    public static function installTables($tables) {

        if(!is_array($tables)) {
          $tables = [$tables];
        }

        foreach($tables as $table) {

            $name = self::getPrefix().'DB_'.$table;
            (new $name())->up();
        }
    }

    public static function removeTables($tables) {

        if(!is_array($tables)) {
          $tables = [$tables];
        }

        foreach($tables as $table) {

            $name = self::getPrefix().'DB_'.$table;
            (new $name())->down();
        }
    }

    public function dropIfExists()
    {
        global $wpdb;

        return $wpdb->query( "DROP TABLE IF EXISTS ".$this->getTableName());

    }

    public function onCreated()
    {
    }

    public function create()
    {

        global $wpdb;

        $charset_collate = $wpdb->get_charset_collate();

        // at the end
        $sql = 'CREATE TABLE '.$this->getTableName().' (';

        $sql .= $this->sql;

        // $sql .= ", PRIMARY KEY  (".$this->primary_key.")
        // ) $charset_collate;";
        $sql .= ", PRIMARY KEY (".$this->primary_key.")) $charset_collate;";

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

        // create initual databse if doesnt exist
        $created = dbDelta( $sql );

        if(!empty($created)) {
          $this->onCreated();
        }


        // add updates if not exists
        $updateCols = $this->addColumns();

        if(!empty($updateCols)) {

          $cols = $this->getTableColumns();

          // check updates
          foreach($updateCols as $key => $value) {

            // if column does not exist, add it
            if(!in_array(key($value), $cols)) {

              $this->addTableColumn($value);
            }
          }

        }

        return true;
    }

    private function addTableColumn($col, $table=null) {

      global $wpdb;

      // convert key value to array
      if(is_array($col)) {
        $key = key($col);
        $col = $key.' '.$col[$key];
      }

      if(empty($table)) {
        $table = $this->getTableName();
      }

      $sql = 'ALTER TABLE '.$table.'  ADD '.$col;
      return $wpdb->query($sql);

    }

    private function getTableColumns($table=null)
    {
      global $wpdb;

      if(empty($table)) {
        $table = $this->getTableName();
      }

      $cols = [];

      foreach ( $wpdb->get_col( "DESC " . $table, 0 ) as $column_name ) {
          array_push($cols, $column_name);
      }

      return $cols;

    }

    // used in update
    public function addColumns() {
      return [];
    }

    public function increments($row='id')
    {
        $this->primary_key = $row;
        $this->sql .= $row.' mediumint(9) NOT NULL AUTO_INCREMENT';
        return $this;
    }

    public function string($row, $length=191)
    {
        $this->sql .= ', '.$row.' varchar('.$length.') NOT NULL';
        return $this;
    }

    public static function stringUpdate($row, $length=191, $nullable=true, $unique=false)
    {
      return self::returnUpdate('varchar', $row, $length, $nullable, $unique);
    }

    public function integer($row, $length=9)
    {
        $this->sql .= ', '.$row.' int('.$length.') NOT NULL';
        return $this;
    }

    public static function bigIntegerUpdate($row, $length=9, $nullable=true, $unique=false)
    {
      return self::returnUpdate('bigint', $row, $length, $nullable, $unique);
    }

    public static function booleanUpdate($row, $length=1, $nullable=true, $unique=false)
    {
      return self::returnUpdate('tinyint', $row, $length, $nullable, $unique);
    }

    public function bigInteger($row, $length=20)
    {
        $this->sql .= ', '.$row.' bigint('.$length.') NOT NULL';
        return $this;
    }

    public function boolean($row)
    {
        $this->sql .= ', '.$row.' tinyint(1) NOT NULL';
        return $this;
    }

    public function decimal($row, $a, $b)
    {
        $this->sql .= ', '.$row.' decimal('.$a.','.$b.') NOT NULL';
        return $this;
    }

    public static function decimalUpdate($row, $a, $b, $nullable=true, $unique=false)
    {
      return self::returnUpdate('decimal', $row, $a.','.$b, $nullable);
    }

    public function double($row, $a, $b)
    {
        $this->sql .= ', '.$row.' double('.$a.','.$b.') NOT NULL';
        return $this;
    }

    public static function doubleUpdate($row, $a, $b, $nullable=true, $unique=false)
    {
      return self::returnUpdate('double', $row, $a.','.$b, $nullable);
    }

    public function float($row, $a, $b)
    {
        $this->sql .= ', '.$row.' float('.$a.','.$b.') NOT NULL';
        return $this;
    }

    public static function floatUpdate($row, $a, $b, $nullable=true, $unique=false)
    {
      return self::returnUpdate('float', $row, $a.','.$b, $nullable);
    }

    public function text($row)
    {
        $this->sql .= ', '.$row.' text NOT NULL';
        return $this;
    }

    public static function textUpdate($row, $nullable=true, $unique=false)
    {
      return [$row => 'text'.($nullable ? '' : ' NOT NULL')];
    }

    public function datetime($row)
    {
        $this->sql .= ', '.$row.' datetime NOT NULL';
        return $this;
    }
    public static function datetimeUpdate($row, $nullable=false, $unique=false)
    {
      return [$row => 'datetime'.($nullable ? '' : ' NOT NULL').($unique ? ' UNIQUE' : '')];
    }

    public function created()
    {
        $this->sql .= ", created datetime NOT NULL";
        return $this;
    }

    public function createdTs()
    {
        return $this->integer('created', 13);
    }

    public function updatedTs()
    {
        return $this->integer('updated', 13);
    }

    public function updated()
    {
        $this->sql .= ", updated datetime NOT NULL";
        return $this;
    }

    public function timestamps()
    {
        $this->sql .= ", created datetime NOT NULL";
        $this->sql .= ", updated datetime NOT NULL";
        return $this;
    }

    public function unsigned()
    {
        $this->sql .= ' UNSIGNED';
        return $this;
    }

    public function nullable()
    {
        $part = explode(',', $this->sql);
        $index = count($part)-1;
        $part[$index] = str_replace(' NOT NULL', '', $part[$index]);
        $this->sql = implode(',', $part);
        return $this;
    }

    public function notnull()
    {
        $this->sql .= ' NOT NULL';
        return $this;
    }

    public function doDefault($default='')
    {
        $this->sql .= " DEFAULT '".$default."'";
        return $this;
    }

    public function unique()
    {
        $this->sql .= ' UNIQUE';
        return $this;
    }



    public function getTableName()
    {
        global $wpdb;

        if($this->use_app_group_id) {

            return $wpdb->prefix . self::$plugin_app_group_id.'_'.$this->table;

        }
        elseif($this->use_app_id) {

            return $wpdb->prefix . self::$plugin_app_id.'_'.$this->table;

        }
        elseif($this->use_custom_id) {

            return $wpdb->prefix . $this->use_custom_id.'_'.$this->table;

        }
        return $wpdb->prefix . $this->table;
    }



    public static function unsignedUpdate($row, $length=9)
    {
      return self::integerUpdate($row, $length, true, false, true);
    }


    public static function integerUpdate($row, $length=9, $nullable=true, $unique=false, $unsigned=false)
    {
      return self::returnUpdate('int', $row, $length, $nullable, $unique, $unsigned);
    }


    private static function returnUpdate($type, $row, $length, $nullable, $unique=false, $unsigned=false) {

      return [$row => $type.'('.$length.')'.($nullable ? '' : ' NOT NULL').($unique ? ' UNIQUE' : '').($unsigned ? ' UNSIGNED' : '')];
    }

}
