<?php

/**
 * https://codex.wordpress.org/Function_Reference/register_post_type
 */

if(class_exists('AffiliatePagesPro_PostType')) {
    return;
}

affiliate_pages_pro_use([
	'Core/Request',

	'Helpers/Admin',
	'Helpers/Form',
	'Helpers/EnqueSimple',
	'Helpers/Options',
	'Helpers/MetaBoxes',
	'Helpers/Nonce',
	'Helpers/Routes',
	'Helpers/Postman',

	// 'Helpers/PostMeta',
	// 'Helpers/Sessions',
	'Helpers/View',
	// 'Helpers/Client'

	'Core/PostType/Create',
	'Core/PostType/Destroy',
	'Core/PostType/Index',
	'Core/PostType/Show',
	'Core/PostType/Update',
	'Core/PostType/Sort',
	'Core/PostType/WP',

	'Helpers/User/UserToken',
	'Helpers/StrGen',

]);

class AffiliatePagesPro_PostType extends AffiliatePagesPro
{
	use AffiliatePagesPro_Admin,
		AffiliatePagesPro_UserToken,
		AffiliatePagesPro_StrGen,
		AffiliatePagesPro_EnqueSimple,
		AffiliatePagesPro_Form,
		AffiliatePagesPro_View,
		AffiliatePagesPro_Options,
		AffiliatePagesPro_MetaBoxes,
		AffiliatePagesPro_Nonce,
		AffiliatePagesPro_Routes,
		AffiliatePagesPro_Postman,
		AffiliatePagesPro_PostTypeCreate,
		AffiliatePagesPro_PostTypeDestroy,
		AffiliatePagesPro_PostTypeIndex,
		AffiliatePagesPro_PostTypeShow,
		AffiliatePagesPro_PostTypeUpdate,
		AffiliatePagesPro_PostTypeSort,
		AffiliatePagesPro_PostTypeWP;

	// Required

	protected $id;

	protected $name;

	protected $singular_name;

	protected $validate_lic = false;

	protected $show_ui = false;


	// protected $metabox_key = 'settings';


	// Additional

	protected $public = false;

	protected $show_in_nav_menus = true;

	protected $show_in_menu = true;

	protected $show_in_admin_bar = true;

	protected $menu_position = 20;

	protected $exclude_from_search = true;

	protected $publicly_queryable = true;

	protected $menu_icon = 'dashicons-format-video';

	protected $capability_type;

	protected $rewrite;

	protected $meta_boxes;

	protected $edit_access_level = 'edit_pages';

	protected $preview_status_message = 'In preview mode. Please publish to load page';

	protected $pending_review_status_message = 'In preview mode. Please publish to load page';

	protected $in_preview_mode = false;

	protected $show_meta_include_empty = true;


	// Advanced custom options

	protected $labels = []; // optionally can be setup

	protected $args = []; // optionally can be setup


	// Design

	protected $styles;// = ['admin'];//['admin', 'air-datepicker'];

	protected $scripts;// = ['admin'];

	protected $js_libs;

	protected $wrapper_class = 'ml-css';

	protected $check_lic = false;


	// Route Datatable

    protected $allow_select = false;

    protected $allow_multiselect = false;

    protected $allow_create = false;

    protected $allow_edit = false;

    protected $allow_delete = false;

    protected $allow_dublicate = false;


    // PostType DataTables

	protected $query_orderby = 'id';

	protected $query_order = 'ASC';

    protected $query_limit = 50; // default limit

    protected $query_page = 1;

    protected $query_post_status = 'any';

    protected $query_meta_key;

    protected $query_meta_value;

    protected $query_meta_compare;

    // get
    protected $query_vars = [
				    	'order',
				    	'orderby',
    					'limit',
    					'page',
    					'post_status',
    					'meta_key',
    					'meta_value',
    					'meta_compare'
    				];

    protected $query_prefix_keys = ['meta_key'];

    protected $query_wp_keys = [
    					'page' => 'paged',
    					'limit' => 'posts_per_page',
    					'q' => 's'
					];

    protected $fillable;

    protected $query_fields = ['id', 'post_title'];

    protected $post_fields = [
    						'post_author',
    						'post_title',
    						'post_content',
    						// 'post_excerpt',
    						'post_status',
    						'comment_status',
    						// 'ping_status',
    						// 'post_password',
    						'post_name',
    						// 'to_ping',
    						// 'pinged',
    						// 'post_content_filtered',
    						'post_parent',
    						// 'guid',
    						'menu_order',
    						// 'post_type',
    						// 'post_mime_type',
    						'comment_count',
    						// 'filter'
    					];



	// alerts

	protected $success;

	protected $error;

	protected $editors; // form editors to save

	protected $meta;

	protected $admin_styles;

	protected $admin_scripts;

	protected $taxonomies;

	public function __construct()
	{
		// append to construct
		$this->onConstruct();

		// Metaboxes
		if(!empty($this->meta_boxes)) {

			add_action( 'add_meta_boxes', [$this, 'metaBoxes'], 10, 2 );

			if($this->nonceValid()) {
				add_action('save_post', [$this, 'metaBoxesOnUpdate']);
			}
		}


		// Register Post Type
		$this->register();

		// Helper Hooks
		add_action('wp', [$this, 'wp']);

		add_action('pre_get_posts', [$this, 'postTypeInit']);

		add_action( 'admin_enqueue_scripts', [$this, 'admin_enqueue_scripts']);

		add_action( 'admin_notices', [$this, 'adminNotice'] );
	}

	public function adminNotice() {
	}

	public function admin_enqueue_scripts()
	{
		global $post;

		if(empty($post) || empty($post->post_type) || $post->post_type != $this->getPostType()) {
			return;
		}

		if(!empty($this->admin_scripts)) {
			$this->enqueSimpleScripts($this->admin_scripts);
		}

		if(!empty($this->admin_styles)) {
			$this->enqueSimpleStyles($this->admin_styles);
		}





	// wp_send_json($post->post_type); exit;
	// 	echo 'enque admin scripts'; exit;
	}



	public function onValidateLic()
	{
	}




	public function postTypeInit() {

		global $post;

		if(empty($post) || empty($post->post_type) || $post->post_type != $this->getPostType()) {
			return;
		}

		if(is_admin()) {


			// $this->lic_form_key = self::$plugin_app_id.'_lic';

			// if(!empty($_POST[$this->lic_form_key])) {
			// 	// validate nonce
			// 	// wp_send_json($_POST[$this->lic_form_key]); exit;
			// 	$this->postLic($_POST[$this->lic_form_key]);

			// 	// wp_send_json($this->error); exit;
			// }

			// $this->lic = $this->getOption('lic');


			// if(!empty($_POST)) {
			// 	wp_send_json($_POST); exit;
			// }
			//

			// admin scripts
			$this->enqueSimpleStyles();
			$this->enqueSimpleScripts();

			if($this->validate_lic) {
				$this->onValidateLic();
			}
		}



	}


	// append to conscruct
	public function onConstruct() {}

	// loads front for all statuses
	public function isPage($request) {}

	public function isPost($request) {}

	// frontend if post set to pending
	public function isDraft() {}
	public function isPending() {}

	public function onCreate($request, $post) {}
	// public function onDelete($request) {}





	public function register() {

		$options = $this->getLabelOptions();

		foreach($options as $i) {
			if(empty($this->labels) || empty($this->labels[$i])) {
				$this->labels[$i] = $this->{$i};
			}
		}

		$args = $this->args;
		$options = $this->getArgOptions();

		foreach($options as $i) {
			if(isset($args[$i]) == false && isset($this->{$i}) == true) {
				$args[$i] = $this->{$i};
			}
		}

		register_post_type($this->getId(), $args);

		// echo $this->getId(); exit;
		if(!empty($this->taxonomies)) {
			$id = $this->getId();
			foreach($this->taxonomies as $key => $value) {
				// echo $id.'_'.$key; exit;
				register_taxonomy(  $id.'_'.$key, $id, $value );
			}
			// foreach($this->taxonomies as $key => value) {
			// 	register_taxonomy(  $id.'_'.$key, $id, $value );
			// }
		}


	}

	public function getPostType($id=null)
	{
		if(empty($id)) {
			$id = $this->id;
		}
		return self::$plugin_app_id.$id;
	}

	public function getPostMeta($postId=null)
	{

		if(empty($postId)) {
			global $post;
			if(empty($post->ID)) {
				return false;
			}
			$postId = $post->ID;
		}

		$key = self::$plugin_app_id.'_'.$this->id;

		$this->post_meta = get_post_meta($postId, $key, true);

		return $this->post_meta;
	}

	// public function getMeta($id=null) {
	// 	$key = $this->getMetaKey($id);

	// 	echo $key; exit;
	// }

	public function getMetaKey($id=null) {
		//echo self::$plugin_app_id.'_'; exit;
		if(empty($id)) {
			$id = $this->id;
		}
		return self::$plugin_app_id.'_'.$id;
	}

	public function getPivotMetaKey($id=null)
	{
		if(empty($id)) {
			global $post; $id = $post->ID;
		}
		return self::$plugin_app_id.'_'.$this->id.'_'.$id;
	}

	private function getId($append='') {
		return self::$plugin_app_id.$this->id.$append;
	}

	private function getLabelOptions()
	{
		return [
			'name',
			'singular_name'
		];
	}

	private function getArgOptions()
	{
		return [
			'labels',
			'public',
			'show_ui',
			'show_in_nav_menus',
			'show_in_menu',
			'show_in_admin_bar',
			'menu_position',
			'exclude_from_search',
			'publicly_queryable',
			'menu_icon',
			'capability_type',
			'rewrite'
		];
	}

}
