<?php

if(trait_exists('AffiliatePagesPro_Helper_Aweber')) {
    return;
}

/**
 *
 * REQUIRES:
 * 'Core/OAuth'
 */

trait AffiliatePagesPro_Helper_Aweber
{

  public $aweber_api_version = '1.0';

  public function aweberGetAccountId($updateIfNew=false)
  {
    if(!empty($this->aweber_account_id)) {
      return $this->aweber_account_id;
    }

    $accountId = $this->getOptionValue('settings', 'aweber_account');

    if(!empty($accountId)) {
      return $accountId;
    }

    // echo 'get account id remote'; exit;

    // do api call to fetch
    $accounts = $this->getAweberAccounts();

    if(empty($accounts) || empty($accounts->entries)) {
      return '';
    }

    $this->aweber_account_id = $accounts->entries{0}->id;

    if(!empty($updateIfNew)) {
      $this->updateOptionValue('settings', 'aweber_account', $this->aweber_account_id);
    }

    return $this->aweber_account_id;

  }

  public function refreshAweberLists($accountId='', $listFormat=true)
  {
    return $this->getAweberLists($accountId, $listFormat=true, true);
  }

  // if no accountId, will look for accounts
  public function getAweberLists($accountId='', $listFormat=true, $refresh=false)
  {

    if(empty($refresh)) {

      $lists = $this->getOption('aweber_lists'); // mailchimp list cache

      if(!empty($lists)) {
        if(!empty($listFormat)) {
          $lists = $this->getAweberListFormat($lists);
        }
        return $lists;
      }

    }


    if(empty($accountId)) {
      $accountId = $this->getOptionValue('settings', 'aweber_account');

      if(empty($accountId)) {

        $accounts = $this->getAweberAccounts();

        if(empty($accounts) || empty($accounts->entries)) {
          $this->error = 'no accounts found';
          return [];
        }

        $accountId = $accounts->entries{0}->id;
      }
    }

    // echo 'accounts/'.$accountId.'/lists'; exit;

    $data = $this->getAweberGet('accounts/'.$accountId.'/lists');

    // wp_send_json($data); exit;

    if(!empty($data) && !empty($data->entries)) {
      $this->updateOption('aweber_lists', $data->entries);

      if(!empty($listFormat)) {
        // wp_send_json($data->entries{0}); exit;
        return $this->getAweberListFormat($data->entries);
      }
      return $data;
    }

    return [];
  }
  //
  public function getAweberListFormat($data)
  {

    $lists = [];

    if(empty($data)) {
      return $lists;
    }

    foreach($data as $item) {

      if(is_object($item)) {


        array_push($lists, ['id' => $item->id, 'name' => $item->name]);
      } else {
        array_push($lists, ['id' => $item['id'], 'name' => $item['name']]);
      }
    }
    return $lists;
  }
  //
  //
  // public function getResponsePost($endpoint='lists/members', $data=null, $returnArray=false, $returnFullResponse=false) {
  //
  //   if(empty($this->aweber_apikey)) {
  //     $this->aweber_apikey = $this->getOptionValue('settings', 'aweber_apikey');
  //   }
  //
  //   if(empty($this->aweber_apikey)) {
  //     $this->error = 'Missing Aweber Api Key';
  //     return;
  //   }
  //
  //   $body = [];
  //
  //   if(!empty($data)) {
  //     // $body = json_decode(json_encode($body));
  //     foreach($data as $key => $value) {
  //         $body[$key]= $value;
  //     }
  //   }
  //
  //   $args = [
  //             'body' => json_encode($body),
  //             'timeout'	=> 60,
  //             'headers' => $this->aweberGetAuth()
  //             ];
  //   $args['headers']['Content-Type'] = 'application/json';
  //   $args['headers']['Accept'] = 'application/json';
  //
  //   // echo $this->aweberGetUrl($endpoint); exit;
  //   // wp_send_json($args); exit;
  //
  //   $response = wp_remote_post($this->aweberGetUrl($endpoint), $args);
  //
  //   if(!empty($returnFullResponse)) {
  //     return $response;
  //   }
  //
  //   // wp_send_json($response['response']['code']); exit;
  //   return json_decode($response['body'], $returnArray);
  //
  // }
  //
  //
  //
  //

  public function getAweberAccounts()
  {
    $data = $this->getOption('aweber_auth');

    if(empty($data) || empty($data['consumer_key'])) {
      $this->error = 'Aweber not setup';
      return []; exit;
    }

    return $this->getAweberGet('accounts');
  }


  public function getAweberGet($endpoint='accounts', $returnArray=false) {

    $data = $this->getOption('aweber_auth');

    if(empty($data) || empty($data['consumer_key'])) {
      $this->error = 'Aweber not setup';
      return []; exit;
    }

		$url = 'https://api.aweber.com/'.$this->aweber_api_version.'/'.$endpoint;

    $oauth = new AffiliatePagesPro_OAuth($data);

    return $oauth->get($url);

  }

  public function aweberPost($endpoint, $args=[], $returnFullResponse=false)
  {
    $data = $this->getOption('aweber_auth');

    if(empty($data) || empty($data['consumer_key'])) {
      $this->error = 'Aweber not setup';
      return []; exit;
    }

    $url = 'https://api.aweber.com/'.$this->aweber_api_version.'/'.$endpoint;

    $oauth = new AffiliatePagesPro_OAuth($data);

    // echo $url; exit;

    return $oauth->post($url, $args, $returnFullResponse);
  }
  //
  // public function aweberGetArgs($body=null) {
  //   return ['body' => $body, 'timeout'	=> 60, 'headers' => $this->aweberGetAuth()];
  // }
  //
  // public function aweberGetAuth() {
  //   return ['X-Auth-Token' => 'api-key '.$this->aweber_apikey];
  //   // return 'Basic '.base64_encode('apiuser:'.$this->aweber_apikey);
  // }
  //
  // public function aweberGetUrl($endpoint='') {
  //   return 'https://api.aweber.com/'.$this->aweber_api_version.'/'.$endpoint;
  // }


}
