<?php

if(trait_exists('AffiliatePagesPro_Helper_MailChimp')) {
    return;
}

/**
 *
 * REQUIRES:
 * 'Core/Uploads',
 * OPTION SET:
 * protected $plugin_build_path (root to where the build will go, default is set below)
 * create $this->pluginbuild_error on error
 */
// protected $pluginbuild_required_fields =

trait AffiliatePagesPro_Helper_MailChimp
{

  public $mailchimp_api_version = '3.0';

  public function getMailChimpLists()
  {

    $lists = $this->getOption('mailchimp_lists'); // mailchimp list cache

    if(!empty($lists)) {
      return $lists;
    }


    // get list
    $data = $this->mailChimpGet('lists');

    if(!empty($data) && !empty($data->lists)) {
      $this->updateOption('mailchimp_lists', $data->lists);
      return $data->lists;
    }

    return [];

  }



  public function mailchimpSubscribe($list, $email, $status='subscribed', $fname='', $lname='', $mergeFields=[], $other=[], $returnArray=false)
  {
    $endpoint = 'lists/'.$list.'/members';

    $data = ['email_address' => $email, 'status' => $status, 'merge_fields' => []];

    if(!empty($fname)) {
      $data['merge_fields']['FNAME'] = $fname;
    }
    if(!empty($lname)) {
      $data['merge_fields']['LNAME'] = $lname;
    }

    // usually merge fields
    if(!empty($mergeFields)) {
      foreach($mergeFields as $key => $value) {
        $data['merge_fields'][$key] = $value;
      }
    }

    // and parse name
    if(!empty($other)) {
      foreach($other as $key => $value) {
        $data[$key] = $value;
      }
    }

    // unset empty merge fields
    if(empty($data['merge_fields'])) {
      unset($data['merge_fields']);
    }

    return $this->mailChimpPost($endpoint, $data, $returnArray);

  }


  public function mailChimpPost($endpoint='lists/members', $data=null, $returnArray=false) {

    if(empty($this->mailchimp_apikey)) {
      $this->mailchimp_apikey = $this->getOptionValue('settings', 'mailchimp_apikey');
    }

    if(empty($this->mailchimp_apikey)) {
      $this->error = 'Missing Mailchimp Api Key';
      return;
    }

    $body = [];

    if(!empty($data)) {
      // $body = json_decode(json_encode($body));
      foreach($data as $key => $value) {
          $body[$key]= $value;
      }
    }

    // echo json_encode($body); exit;

    $args = [
              'body' => json_encode($body),
              'timeout'	=> 60,
              'headers' => [
                  'Authorization' => $this->mailchimpGetAuth(),
                  'Content-Type' => 'application/json',
                  'Accept'     => 'application/json',
                ],

              ];

    // wp_send_json($args); exit;
    // echo $this->mailchimpGetUrl($endpoint); exit;

    $response = wp_remote_post($this->mailchimpGetUrl($endpoint), $args);

    return json_decode($response['body'], $returnArray);

  }




  public function mailChimpGet($endpoint='lists', $returnArray=false) {

    if(empty($this->mailchimp_apikey)) {
      $this->mailchimp_apikey = $this->getOptionValue('settings', 'mailchimp_apikey');
    }

    if(empty($this->mailchimp_apikey)) {
      $this->error = 'Missing Mailchimp Api Key';
      return;
    }

    $response = wp_remote_get($this->mailchimpGetUrl($endpoint), $this->mailchimpGetArgs());

    return json_decode($response['body'], $returnArray);

  }

  public function mailchimpGetArgs($body=null) {
    return ['body' => $body, 'timeout'	=> 60, 'headers' => ['Authorization' => 'Basic '.base64_encode('apiuser:'.$this->mailchimp_apikey)]];
  }

  public function mailchimpGetAuth() {
    return 'Basic '.base64_encode('apiuser:'.$this->mailchimp_apikey);
  }

  public function mailchimpGetUrl($endpoint='') {
    return 'https://'.$this->mailchimpGetDC().'.api.mailchimp.com/'.$this->mailchimp_api_version.'/'.$endpoint;
  }

  public function mailchimpGetDC($key=null) {
    if(empty($key)) {
      $key = $this->mailchimp_apikey;
    }
    $part = explode('-', $key);
    return array_pop($part);
  }






  // Snippets

  public function mailchimpGetSubscriptionStatusOptions() {
    return [
      [
      'id' => 'subscribed',
      'name' => 'Subscribed',
      'des' => 'This address is on the list and ready to receive email. You can only send campaigns to ‘subscribed’ addresses.'
    ],
      [
      'id' => 'unsubscribed',
      'name' => 'Unsubscribed',
      'des' => 'This address used to be on the list but isn’t anymore.'
    ],
      [
      'id' => 'pending',
      'name' => 'Pending',
      'des' => 'This address requested to be added with double-opt-in but hasn’t confirmed their subscription yet.'
    ],
      [
      'id' => 'cleaned',
      'name' => 'Cleaned',
      'des' => 'This address bounced and has been removed from the list.'
    ]
  ];
  }
}
