<?php

if(trait_exists('AffiliatePagesPro_Token')) {
    return;
}

/**
 * Requires: Helpers/StrGen
 *
 * Creates a unique token and stores it in the options database with timestamp
 * expire can be validated
 */

trait AffiliatePagesPro_Token
{
	protected $token_db_sep = '|';

	// optional: protected $token_expires = 60; // minutes 
	// optional: protected $token_expires_days = 30; // days
	// optional: protected $token_expires_in = SECONDS; // minutes
	// optional: protected $token_length = 10;

	/**
	 * @param string, the name of the token
	 * @param  string, token to be validated
	 * @param  integer, expire in minutes from creation
	 * @return boolean, true if valid, false if no token or invalid
	 */
	public function validateToken($name, $token=null, $delete=false)
	{
		if(empty($token)) {
			return false;
		}

		$token = $this->getToken($name, true);

		if($delete) {
			$this->deleteToken($name);
		}

		if(!empty($token['expires_in'])) {
			$expiresIn = $token['expires_in'];
		}
		elseif(!empty($this->token_expires_in)) {
			$expiresIn = $this->token_expires_in;			
		}
		elseif(!empty($this->token_expires)) {
			$expiresIn = $this->token_expires * 60;			
		}
		elseif(!empty($this->token_expires_days)) {
			$expiresIn = $this->token_expires_days * 86400;		
		}

		if(!empty($expiresIn)) {

			$expires = $token['created']+$expiresIn;
			
			if(current_time('U') > $expires) {
				return false;
			}
		}

		if($token['token'] === $token['token']) {
			return true;
		}

		return false;


	}

	public function deleteToken($name)
	{
		$key = $this->getTokenKey($name);
		$this->deleteOption($key);
	}

	public function getValidToken($name, $returnArray=false)
	{
		$token = $this->getToken($name, true);

		if($this->validateToken($name, $token)) {
			if($returnArray) {
				return $token;
			} else {
				return $token['token'];
			}
		}

		// not valid

	}

	/**
	 * @param  string, gets token
	 * @return string, returns token
	 */
	public function getToken($name, $returnArray=false, $delete=false)
	{
		$key = $this->getTokenKey($name);
		$data = $this->getOption($key);
		if(empty($data)) {
			return;
		}

		if($delete) {
			$this->deleteOption($key);
		}
		
		$part = explode($this->token_db_sep, $data);

		if($returnArray) {
			$re = ['token' => $part[1], 'created' => $part[0]];
			if(!empty($part[2])) {
				$re['expires_in'] = $part[2];
			}
			return $re;
		}

		return $part[1];
	}

	/**
	 * @param  String, name of the token
	 * @param  string actual token
	 * @return string, token
	 */
	public function updateToken($name, $token, $expiresIn = null, $returnToken = false)
	{
		$key = $this->getTokenKey($name);
		$value = current_time('U').$this->token_db_sep.$token;
		if(!empty($expiresIn)) {
			$value .= $this->token_db_sep.$expiresIn;
		}
		$this->updateOption($key, $value);
		if($returnToken) {
			return $token;
		}
	}
	
	/**
	 * @param  string
	 * @param  integer
	 * @return string token
	 */
	public function createToken($name, $length = 255)
	{
		// optional overrite
		if($this->token_length) {
			$length = $this->token_length;
		}

		$key = $this->getTokenKey($name);
		$token = $this->strGenUnique($length);
		$value = current_time('U').$this->token_db_sep.$token;

		$this->updateOption($key, $value);

		return $token;
	}

	public function getTokenKey($name) {
		return str_replace(' ', '_', strtolower($name)).'_token';
	}

	// Add here user token
}