<?php

if(trait_exists('AffiliatePagesPro_UserLogin')) {
    return;
}

// requires userToken helper


trait AffiliatePagesPro_UserLogin
{



	public function createUserLoginToken($userId)
	{
		$this->user_token_key = 'login_token';
		return $this->getOrCreateUserToken($userId);
	}

	// authenticate login 
	public function doUserLoginWithToken($request, $remember = true, $secure = '', $token = '')
	{

		if(!$request->validate(['login_token', 'user_id'])) {
			return;
		}

		$this->user_token_key = 'login_token';
		$this->user_token_expire = 3600;

		$isValid = $this->validateAndRemoveUserToken($request->login_token, $request->user_id);

		if(!empty($isValid)) {

			// Sets Redirect Login AAAAAA
			wp_set_auth_cookie($request->user_id, $remember, $secure, $token);

			// if success_return_url redirect set
			if(!empty($request->success_return_url)) {
				wp_redirect($request->success_return_url); exit;
			}

		} 

		// if failed_return_url redirect set
		elseif(!empty($request->failed_return_url)) {
			wp_redirect($request->failed_return_url); exit;
		}
		
		// if return_url redirect set
		if(!empty($request->return_url)) {
			wp_redirect($request->return_url); exit;
		}

		return $isValid;
	}

	// createUserLoginWithToken


	public function userTokenLogin($login, $password, $remember = true, $secure_cookie='')
	{
		if (empty($login) || !is_string($login)) {
			$this->error = 'login'; $this->error_message = 'Enter a username or email address';
			return false;

		} 


		// OPTION A

		// $user = wp_signon([
		// 	'user_login' => $login, 
		// 	'user_password' => $password, 
		// 	'remember' => $remember
		// 	], $secure_cookie);


		// OPTION B

		if(strpos($login, '@')) {
			$user = get_user_by('email', trim(wp_unslash($login)));
			// $user = $userData->userLogin;

		}
		else {
			$user = get_user_by('login', trim($login));
		}

		if(empty($user) || empty($user->ID)) {
			return null;
		}

		$username = sanitize_user($login);
    	$password = trim($password);

    	$user = apply_filters( 'authenticate', null, $username, $password);


		// wp_send_json($signon->data->user_login); exit;

		if(empty($user) || empty($user->ID)) {
			return false;
		}

		$this->user_token_key = 'login_token';


		$token = $this->getOrCreateUserToken($user->ID);

		return ['login_token' => $token, 'user_id' => $user->ID, 'login_success' => true];
	}


	/**
	 * logs user in
	 * @param  string $username email or username
	 * @param  [type] $password [description]
	 * @return [type]           [description]
	 */
	public function userLogin($user, $password, $remember = true, $secure_cookie='')
	{
		if (empty($user) || !is_string($user)) {
			$this->error = 'user'; $this->error_message = 'Enter a username or email address';
			return false;

		} 

		$signon = wp_signon([
			'user_login' => $user, 
			'user_password' => $password, 
			'remember' => $remember
			], $secure_cookie);

		// wp_send_json($signon->data->user_login); exit;

		if(empty($signon) || empty($signon->ID)) {
			return false;
		}

		// successfully signin in
		wp_set_current_user($signon->ID, $signon->data->user_login);
		wp_set_auth_cookie($signon->ID, $remember, $secure_cookie);

		return $signon->ID;
		// wp_send_json($signon->ID); exit;
	}

	public function userLoginHelp($user) {

		$errors = new WP_Error();

		if (empty($user) || !is_string($user)) {
			$this->error = 'user'; $this->error_message = 'Enter a username or email address';
			return false;

		} elseif(strpos($user, '@')) {
			$userData = get_user_by('email', trim(wp_unslash($user)));
			$user = $userData->userLogin;

		}
		else {
			$userData = get_user_by('login', trim($user));
		}
		
		do_action( 'lostpassword_post', $errors );

		if ($errors->get_error_code()) {
			$this->error_code = $errors->get_error_code();
			return false;
		}

		if (empty($userData)) {
			$errors->add('invalidcombo', __('<strong>ERROR</strong>: Invalid username or email.'));
			$this->error = 'not_found'; $this->error_message = 'That account does not exist';
			return false;
		}


		// Redefining user_login ensures we return the right case in the email.
		$userLogin = $userData->userLogin;
		$userEmail = $userData->user_email;
		$key = get_password_reset_key($userData);

		if (is_wp_error($key)) {
			$this->error = 'reset_key'; $this->wp_key = $key;
			return false;
		}

		if ( is_multisite() ) {
			$siteName = get_network()->site_name;
		} else {
			$siteName = wp_specialchars_decode(get_option('blogname'), ENT_QUOTES);
		}

		$message = __( 'Someone has requested a password reset for the following account:' ) . "\r\n\r\n";
		
		$message .= sprintf( __( 'Site Name: %s'), $siteName ) . "\r\n\r\n";
		
		$message .= sprintf( __( 'Username: %s'), $userLogin ) . "\r\n\r\n";
		$message .= __( 'If this was a mistake, just ignore this email and nothing will happen.' ) . "\r\n\r\n";
		$message .= __( 'To reset your password, visit the following address:' ) . "\r\n\r\n";
		$message .= '<' . network_site_url( "wp-login.php?action=rp&key=$key&login=" . rawurlencode( $userLogin ), 'login' ) . ">\r\n";

		
		$title = sprintf( __('[%s] Password Reset'), $siteName);

		$title = apply_filters('retrieve_password_title', $title, $userLogin, $user_data);

		$message = apply_filters('retrieve_password_message', $message, $key, $userLogin, $user_data);

		if ($message && !wp_mail($userEmail, wp_specialchars_decode($title), $message)) {
			$this->error = 'sending'; $this->error_message = 'The email could not be sent. (Possible reason: your host may have disabled the mail() function.)';
			return false;
		}

		return true;
	}


}



