<?php

if(trait_exists('AffiliatePagesPro_UserLoginRecovery')) {
    return;
}

// requires userToken helper


trait AffiliatePagesPro_UserLoginRecovery
{

	protected $login_recovery_token_key = 'login_recovery_token';

	protected $login_recovery_token_expire = 7200;

	protected $login_recovery_token_length = 64;

	protected $login_recovery_token_invalid; // will be set to true if invalid

	/**
	 * will remove logged in user if no userID passed
	 * @param  [type] $userId [description]
	 * @return [type]         [description]
	 */
	public function removeRecoveryToken($userId=null)
	{

		$this->user_token_key = $this->login_recovery_token_key;
		return $this->removeUserToken($userId);
	}

	public function validateRecoverToken($token, $userString, $logUserIn=false, $logUserInRemember = true, $logUserInSecure = '', $logUserInToken = '')
	{

		// echo $token; exit;

		if(strstr($userString, '@') == true) {
			$user = get_user_by('email', $userString);
		} else {
			$user = get_user_by('login', $userString);
		}

		if(empty($user) || empty($user->ID)) {
			$this->error = 'invalid_user';
			return false;
		}

		$userId = $user->ID;

		// echo $userId; exit;

		if(!empty($logUserIn)) {
			wp_set_auth_cookie($userId, $logUserInRemember, $logUserInSecure, $logUserInToken);
		}

		$this->user_token_key = $this->login_recovery_token_key;
		$this->user_token_expire = $this->login_recovery_token_expire;

		$isValid = $this->validateUserToken($token, $userId, false);

		// wp_send_json(['user' => $userId, 'token' => $token, 'isValid' => $isValid]); exit;

		if(!empty($isValid)) {
			return $user;
		}
		return false;
	}

	public function createRecoveryToken($userId)
	{
		$this->user_token_key = $this->login_recovery_token_key;

		$this->user_token_expire = $this->login_recovery_token_expire;

		$token = $this->createUserToken($userId, $this->login_recovery_token_length);

		return $token;
	}























	public function validateUserLoginRecoveryToken($request)
	{
		if(!$request->validate(['login_recovery_token', 'user'])) {
			return false;
		}

		$user = get_user_by('login', $request->user);

		if(empty($user) || empty($user->ID)) {
			$this->error = 'invalid_user';
			return false;
		}

		$this->user_token_key = $this->login_recovery_token_key;
		$this->user_token_expire = $this->login_recovery_token_expire;

		echo $request->login_recovery_token; exit;


		return $this->validateUserToken($request->login_recovery_token, $user->ID);
	}

	/**
	 * User
	 * @param  [type] $postId [description]
	 * @param  string $user   username
	 * @param  [type] $token  [description]
	 * @return [type]         [description]
	 */
	public function getUserLoginRecovery($postId, $user, $token)
	{
		$url = get_permalink($postId);

		$url = $this->urlAppend($url, ['user' => $user, 'login_recovery_token' => $token]);

		return $url;
	}

	/**
	 * validate recovery token, and set password. delete token
	 * @param  [type] $request [description]
	 * @return [type]          [description]
	 */
	public function userRecoverySetPass($request, $deleteToken=true)
	{
		if(!$request->validate(['login_recovery_token', 'user', 'password'])) {
			return false;
		}

		$user = get_user_by('login', $request->user);

		if(empty($user) || empty($user->ID)) {
			$this->error = 'invalid_user';
			return false;
		}

		$this->user_token_key = $this->login_recovery_token_key;
		$this->user_token_expire = $this->login_recovery_token_expire;

		$isValid = $this->validateUserToken($request->login_recovery_token, $user->ID);

		wp_send_json(['success' => 1, 'message' => $isValid]); exit;

		if(empty($isValid)) {
			$this->error = 'invalid_token';
			return false;
		}

		if($isValid && !empty($deleteToken)) {
			// delete token
			// $this->removeUserToken($user->ID);
		}

		// set user password
		wp_set_password( $request->password, $user->ID);

		return $isValid;
	}


	public function userLoginRecovery($request, $remember = true, $secure = '', $token = '')
	{
		if(!$request->validate(['login_recovery_token', 'user'])) {
			return;
		}

		$user = get_user_by('login', $request->user);

		if(empty($user) || empty($user->ID)) {
			return;
		}

		$this->user_token_key = $this->login_recovery_token_key;
		$this->user_token_expire = $this->login_recovery_token_expire;

		$isValid = $this->validateUserToken($request->login_recovery_token, $user->ID);
		// $isValid = $this->validateAndRemoveUserToken($request->login_recovery_token, $user->ID);
		//echo $isValid; exit;

		if(!empty($isValid)) {
			wp_set_auth_cookie($user->ID, $remember, $secure, $token);

			// if success_return_url redirect set
			if(!empty($request->success_return_url)) {
				wp_redirect($request->success_return_url); exit;
			}

		}

		// if failed_return_url redirect set
		elseif(!empty($request->failed_return_url)) {
			wp_redirect($request->failed_return_url); exit;
		}

		else {
			$this->login_recovery_token_invalid = true;
		}

		// if return_url redirect set
		if(!empty($request->return_url)) {
			wp_redirect($request->return_url); exit;
		}

		// returns user if logged in
		if(!empty($isValid)) {
			return $user;
		}

		return $isValid;
	}





	public function createUserLoginRecoveryToken($userId)
	{
		$this->user_token_key = $this->login_recovery_token_key;

		$this->user_token_expire = $this->login_recovery_token_expire;

		$token = $this->createUserToken($userId, $this->login_recovery_token_length);

		return $token;
	}
}
