<?php

if(trait_exists('AffiliatePagesPro_UserToken')) {
    return;
}

/**
 * Requires: Helpers/StrGen
 *
 * Creates a unique token and stores it in the options database with timestamp
 * expire can be validated
 */

trait AffiliatePagesPro_UserToken
{
	protected $user_token_cache = null;

	protected $user_token_key = 'user_token';

	protected $user_token_length = 64;

	protected $user_token_expire = 86400; // 86400: 60 days // minutes1440

	public function onAdminApiAuth($request)
	{
		$userId = null;
		if(!empty($request->user_id)) {
			$userId = $request->user_id;
		}
		$valid = $this->validateUserToken($request->user_token, $userId);

		if(empty($valid)) {
			wp_send_json_error('invalid_user_token'); exit;
		}
	}

	// use if valid one exists
	public function getOrCreateUserToken($userId = null)
	{

		if(!empty($this->user_token_cache)) {
			return $this->user_token_cache['token'];
		}

		if(empty($userId)) {
			$userId = get_current_user_ID();
		}

		$lookup = $this->getUserTokenWithCache($userId);//get_user_meta($userId, $this->getUserTokenKey(), true);

		if(empty($lookup) || $this->userTokenExpired($lookup['ts'])) {
			return $this->createUserToken($userId);
		}

		$this->user_token_cache = $lookup;
		return $lookup['token'];

	}

	/**
	 * creates and returns a new user token
	 * @param  [type] $userId [description]
	 * @return [type]         [description]
	 */
	public function createUserToken($userId = null, $length=null)
	{
		if(empty($length)) {
			$length = $this->user_token_length;
		}

		if(empty($userId)) {
			$userId = get_current_user_ID();
		}

		$token = [
			'token' => $this->strGen($length),
			'ts' => date('U')
		];

		update_user_meta($userId, $this->getUserTokenKey(), $token);
		$this->user_token_cache = $token;

		return $token['token'];
	}

	/**
	 * Validates user token
	 * @param  [type] $token  [description]
	 * @param  [type] $userId [description]
	 * @return [type]         [description]
	 */
	public function validateUserToken($token, $userId=null, $cached=true)
	{
		if(!empty($this->user_id) && empty($userId)) {
			$userId = $this->user_id;
		}

		if(empty($userId)) {
			$userId = get_current_user_ID();
		}

		if(!empty($cached)) {
			$lookup = $this->getUserTokenWithCache($userId);
		} else {
			$lookup = $this->getUserToken($userId);
		}
		//get_user_meta($userId, $this->getUserTokenKey(), true);
		//if($lookup['token'] != $token) { echo $userId.', '.$token.' - '.$lookup['token']; }; exit;

		// if($lookup['token'] != $token) {
		// 	// echo 'no match ';
		// }
		// echo $this->userTokenExpired($lookup['ts']); exit;

		if(empty($lookup) || $this->userTokenExpired($lookup['ts']) || $lookup['token'] != $token) {
			return false;
		}

		return true;

	}


	public function validateAndRemoveUserToken($token, $userId=null)
	{
		if(!empty($this->user_id) && empty($userId)) {
			$userId = $this->user_id;
		}

		$isValid = $this->validateUserToken($token, $userId);

		if(!empty($isValid)) {
			// delete
			$this->removeUserToken($userId);

		}

		return $isValid;
	}

	public function removeUserToken($userId=null)
	{
		if(empty($userId)) {
			$userId = get_current_user_ID();
		}

		delete_user_meta($userId, $this->getUserTokenKey());
		if(!empty($this->user_token_cache)) {
			$this->user_token_cache = null;
		}
	}

	/**
	 * checks if user token is expired
	 * @param  [type] $ts [description]
	 * @return [type]     [description]
	 */
	private function userTokenExpired($ts) {
		$expire = $this->user_token_expire + $ts;

		$expires = (($this->user_token_expire*60) + $ts);

		if(date('U') > (($this->user_token_expire*60) + $ts)) {
			return true;
		}
		return false;

		// echo 'check if expired expires: '.$expires; exit;

		return (date('U') > (($this->user_token_expire*60) + $ts) == true);
	}

	/**
	 * gets existing user token
	 * @return [type] [description]
	 */
	private function getUserTokenKey()
	{
		return self::$plugin_app_id.'_'.$this->user_token_key;
	}

	private function getUserTokenWithCache($userId) {

		if(!empty($this->user_token_cache)) {
			return $this->user_token_cache;
		}

		$this->user_token_cache = get_user_meta($userId, $this->getUserTokenKey(), true);

		return $this->user_token_cache;

	}

	private function getUserToken($userId) {

		return get_user_meta($userId, $this->getUserTokenKey(), true);

	}
}
